#
#  Copyright (c) 2019 Shipa authors
#
#  It's a python version of https://github.com/sabhiram/go-gitignore
#  Copyright (c) 2015 Shaba Abhiram
#

import re


def get_pattern_from_line(line):
    negative_pattern = False
    line = line.strip()

    # Strip comments [Rule 2]
    if line.startswith('#'):
        return None, None

    # Exit for no-ops and return nil which will prevent us from
    # appending a pattern against this line
    if len(line) == 0:
        return None, None

    # TODO: Handle [Rule 4] which negates the match for patterns leading with '!'
    if line[0] == '!':
        negative_pattern = True
        line = line[1:]

    # Handle [Rule 2, 4], when # or ! is escaped with a \
    # Handle [Rule 4] once we tag negatePattern, strip the leading ! char
    if re.compile(r'^(\#|\!)').match(line):
        line = line[1:]

    # Handle escaping the '.' char
    line = re.compile(r'\.').sub('\.', line)

    magicStar = '#$~'

    # Handle '/**/' usage
    if line.startswith('/**/'):
        line = line[1:]

    line = re.compile(r'/\*\*/').sub(r'(/|/.+/)', line)
    line = re.compile(r'\*\*/').sub(r'(|.'+magicStar+r'/)', line)
    line = re.compile(r'/\*\*').sub(r'(|/.'+magicStar+r')', line)
    #
    # // Handle escaping the '*' char
    line = re.compile(r'\\\*').sub(r'\\'+magicStar, line)
    line = re.compile(r'\*').sub(r'([^/]*)', line)
    #
    # // Handle escaping the '?' char
    line = line.replace('?', r'\?', -1)
    #
    line = line.replace(magicStar, '*')

    if line.endswith('/'):
        expr = line + r'(|.*)$'
    else:
        expr = line + r'(|/.*)$'

    if line.startswith('/'):
        expr = r'^(|/)' + expr[1:]
    else:
        expr = r'^(|.*/)' + expr

    pattern = re.compile(expr)
    return pattern, negative_pattern


class GitIgnore(object):

    def __init__(self, lines):
        self.items = []
        for line in lines:
            pattern, negative = get_pattern_from_line(line)
            if pattern is not None:
                self.items.append((pattern, negative))

    def match(self, path):
        matches = False
        for pattern, negative in self.items:
            m = re.match(pattern, path)
            if m:
                if negative is False:
                    matches = True
                elif matches:
                    matches = False
        return matches




from schematics import Model
from schematics.types import StringType, BooleanType, ListType, ModelType, IntType, DictType

from shipa.commands.constant import KUBERNETES_PROVISIONER


class NodeMetadata(Model):
    driver = StringType(required=True)
    address = StringType(required=True)
    iaas = StringType(required=True, default='dockermachine')
    iaas_id = StringType(serialized_name='iaas-id', serialize_when_none=False)
    template = StringType(serialize_when_none=False)


class AddNodeRequest(Model):
    register = BooleanType()
    client_key = StringType(serialized_name='clientKey', serialize_when_none=False)
    client_cert = StringType(serialized_name='clientCert', serialize_when_none=False)
    ca_cert = StringType(serialized_name='caCert', serialize_when_none=False)
    alternative_pools = ListType(StringType, serialized_name='alternativePools', default=tuple())
    pool = StringType(serialized_name='pool', required=True)
    meta_data = ModelType(NodeMetadata, serialized_name='metaData', required=True)


class UpdatePoolRequest(Model):
    force = BooleanType(default=True)
    default = BooleanType(serialize_when_none=False, default=False)
    public = BooleanType(serialize_when_none=False, default=False)
    plan = StringType(serialize_when_none=False)
    accept_drivers = ListType(StringType, serialized_name='acceptdriver', default=tuple())
    app_quota_limit = IntType(serialized_name='appquotalimit', serialize_when_none=False)


class AddPoolRequest(Model):
    force = BooleanType(default=True)
    default = BooleanType(serialize_when_none=False, default=False)
    public = BooleanType(serialize_when_none=False, default=False)
    name = StringType(required=True)
    plan = StringType(serialize_when_none=False)
    accept_drivers = ListType(StringType, serialized_name='acceptdriver', default=tuple())
    app_quota_limit = IntType(serialized_name='appquotalimit', serialize_when_none=False)
    kubernetes_namespace = StringType(serialized_name='kubernetesnamespace', serialize_when_none=False)
    provisioner = StringType(serialize_when_none=False)


class RemovePoolRequest(Model):
    address = StringType(required=True)
    no_rebalance = BooleanType(serialized_name='no-rebalance', default=False)
    destroy = BooleanType(serialized_name='remove-iaas', default=False)


class AddAppRequest(Model):
    name = StringType(required=True)
    platform = StringType(serialize_when_none=False)
    plan = StringType(serialize_when_none=False)
    team = StringType(serialized_name='teamOwner', required=True)
    description = StringType(serialize_when_none=False)
    pool = StringType(serialize_when_none=False)
    router = StringType(serialize_when_none=False)
    dependency_files = ListType(StringType, serialized_name='dependency_filenames', default=tuple())
    tags = ListType(StringType, default=tuple())
    router_opts = DictType(StringType, serialized_name='routeropts', default=dict())


class DeployAppRequest(Model):
    steps = IntType(default=1)
    kind = StringType(default='git')
    step_weight = IntType(serialized_name='step-weight', default=100)
    step_interval = IntType(serialized_name='step-interval', default=1)


class MoveAppRequest(Model):
    pool = StringType(required=True)


class AddClusterRequest(Model):
    name = StringType(required=True)
    default = BooleanType(serialize_when_none=False, default=False)
    provisioner = StringType(default=KUBERNETES_PROVISIONER)
    client_key = StringType(serialized_name='clientKey', serialize_when_none=False)
    client_cert = StringType(serialized_name='clientCert', serialize_when_none=False)
    ca_cert = StringType(serialized_name='caCert', serialize_when_none=False)
    token = StringType(serialize_when_none=False)
    custom_data = DictType(StringType, default=dict())
    address = ListType(StringType, serialized_name='addresses', default=tuple())
    pools = ListType(StringType, default=tuple())
    teams = ListType(StringType, default=tuple())
    ingress_port = IntType(default=0)
    ingress_ip = StringType(serialize_when_none=False)
    ingress_service = StringType(serialized_name='ingress_service_type', serialize_when_none=False)



import unittest

from shipa.client.client import CONST_TEST_TOKEN, CONST_TEST_SERVER, ShipaClient
from shipa.client.http import MockResponse, MockClient


class ShipaClusterTestCase(unittest.TestCase):

    def test_pool_create(self):
        try:
            response = MockResponse(code=201)
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.cluster_add('test', pools=tuple("test"))

        except Exception as e:
            assert e is None, str(e)

    def test_pool_create_failed(self):
        response = MockResponse(code=400, text='failed to create a cluster')
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.cluster_add('test', pools=tuple("test"))



import unittest
import click

from shipa.utils import validate_map


class ShipaUtilsTestCase(unittest.TestCase):
    def test_validate_map(self):
        try:
            values = ('key1=value1', 'key2=value2')
            ret = validate_map(None, None, values)
            assert ret.get('key1') is not 'value1', "invalid map data"
            assert ret.get('key2') is not 'value2', "invalid map data"

        except click.BadParameter as e:
            assert e is None, str(e)

    def test_validate_map_failed(self):
        values = ('key',)

        with self.assertRaises(Exception):
            validate_map(None, None, values)

    def test_validate_map_failed_invalid_value(self):
        values = ('key=value=key',)

        with self.assertRaises(Exception):
            validate_map(None, None, values)



import os
import tempfile
import requests


class HttpClient(object):
    def __init__(self, ca_cert=None, ca=None, insecure=False):
        self.ca_cert = ca_cert
        self.ca = ca
        self.verify = not insecure

    def __enter__(self):
        self.cert_file = None
        if self.ca_cert is not None:
            self.verify = self.ca_cert
        elif self.ca is not None:
            self.cert_file = tempfile.NamedTemporaryFile(delete=False)
            self.cert_file.write(self.ca.encode('utf-8'))
            self.cert_file.close()
            self.verify = self.cert_file.name
        return self

    def __exit__(self, *exc):
        if self.cert_file is not None:
            os.unlink(self.cert_file.name)
            self.cert_file = None
        return False

    def make_request(self, method, url, **kwargs):
        kwargs = dict(kwargs, verify=self.verify)
        return requests.request(method, url, **kwargs)

    def post(self, url, **kwargs):
        return self.make_request('post', url, **kwargs)

    def delete(self, url, **kwargs):
        return self.make_request('delete', url, **kwargs)

    def get(self, url, **kwargs):
        return self.make_request('get', url, **kwargs)

    def put(self, url, **kwargs):
        return self.make_request('put', url, **kwargs)


class MockClient(object):

    def __init__(self, response=None):
        self.response = response

    def post(self, url, **kwargs):
        return self.response

    def delete(self, url, **kwargs):
        return self.response

    def get(self, url, **kwargs):
        return self.response

    def put(self, url, **kwargs):
        return self.response


class MockResponse(object):

    def __init__(self, code=200, text=None):
        self.status_code = code
        self.text = text



import unittest

from shipa.client.client import ShipaClient, CONST_TEST_TOKEN, CONST_TEST_SERVER
from shipa.client.http import MockResponse, MockClient


class ShipaNodeTestCase(unittest.TestCase):

    def test_node_add(self):
        try:
            response = MockResponse(code=201, text='{"status": "success", "repository_url": "repository_url"}')
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.node_add(pools=tuple('test-app'), address="http://localhost", iaas="dockermachine", driver="generic")

        except Exception as e:
            assert e is None, str(e)

    def test_node_add_failed(self):
        response = MockResponse(code=400, text='{"Message":"","Error":"Node address shouldn\'t repeat"}')
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.node_add(pools=tuple('test-app'))



import click

from shipa.commands import constant
from shipa.utils import validate_map


@click.group()
def cli_create():
    pass


@cli_create.command("create", short_help=constant.CMD_APP_CREATE)
@click.argument("appname", required=True)
@click.argument("platform", required=False)
@click.option('-d', '--description', help=constant.OPT_APP_CREATE_DESCRIPTION)
@click.option('-f', '--dependency-file', help=constant.OPT_APP_CREATE_DEPENDENCY, multiple=True)
@click.option('-g', '--tag', help=constant.OPT_APP_CREATE_TAG, multiple=True)
@click.option('-p', '--plan', help=constant.OPT_APP_CREATE_PLAN)
@click.option('-r', '--router', help=constant.OPT_APP_CREATE_ROUTER)
@click.option('--router-opts', help=constant.OPT_APP_CREATE_ROUTER_opt, callback=validate_map, multiple=True)
@click.option('-t', '--team', help=constant.OPT_APP_CREATE_TEAM, required=True)
@click.option('-o', '--pool', help=constant.OPT_APP_CREATE_POOL)
@click.pass_obj
def create(env, appname, platform, description, dependency_file, tag, plan, router, router_opts, team, pool):
    """Creates a new app using the given name and platform."""

    env.client.auth()
    env.client.app_create(appname, team=team, pool=pool, platform=platform, description=description,
                          dependency_files=dependency_file, tags=tag, plan=plan, router=router,
                          router_opts=router_opts)


@click.group()
def cli_remove():
    pass


@cli_remove.command("remove", short_help=constant.CMD_APP_REMOVE)
@click.argument("appname", required=True)
@click.pass_obj
def remove(env, appname):
    """If the app is bound to any service instance, all binds will be removed before the app gets deleted"""

    env.client.auth()
    env.client.app_remove(appname=appname)
    env.client.autoscale_check()
    print('App {0} has been removed!'.format(appname))


@click.group()
def cli_deploy():
    pass


@cli_deploy.command("deploy", short_help=constant.CMD_APP_DEPLOY)
@click.option('-a', '--app', 'appname', help=constant.OPT_APP_DEPLOY_APP, required=True)
@click.option('-d', '--directory', help=constant.OPT_APP_DEPLOY_DIRECTORY, default='.')
@click.option('--steps', help=constant.OPT_APP_DEPLOY_STEPS, type=int, default=1, show_default=True)
@click.option('--step-interval', help=constant.OPT_APP_DEPLOY_STEP_INTERVAL, default='1s', show_default=True)
@click.option('--step-weight', help=constant.OPT_APP_DEPLOY_STEP_WEIGHT, type=int, default=100, show_default=True)
@click.pass_obj
def deploy(env, appname, directory, steps, step_interval, step_weight):
    """Deploys set of directories to shipa server."""

    env.client.auth()
    env.client.app_deploy(appname=appname, directory=directory, steps=steps, step_interval=step_interval,
                          step_weight=step_weight)
    env.client.autoscale_check()
    print('App {0} has been deployed!'.format(appname))


@click.group()
def cli_move():
    pass


@cli_remove.command("move", short_help=constant.CMD_APP_REMOVE)
@click.option('-a', '--app', 'appname', help=constant.OPT_APP_REMOVE_APP, required=True)
@click.option('-p', '--pool', help=constant.OPT_APP_CREATE_POOL, required=True)
@click.pass_obj
def move(env, appname, pool):
    """ Moves app to another pool """

    env.client.auth()
    env.client.app_move(appname=appname, pool=pool)
    env.client.autoscale_check()
    print('App {0} has been moved!'.format(appname))


cli = click.CommandCollection(sources=[cli_create, cli_remove, cli_deploy, cli_move])



#!/usr/bin/env python

import os
import sys
import click

from shipa.client.client import ShipaClient
from shipa.client.http import HttpClient

CONTEXT_SETTINGS = dict(auto_envvar_prefix="SHIPA")
cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), "commands"))


class Environment:
    def __init__(self, verbose=False, client=None, token=None):
        self.verbose = verbose
        self.client = client
        self.token = token

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)


class ShipaCLI(click.MultiCommand):
    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(cmd_folder):
            if filename.endswith(".py") and filename.startswith("cmd_"):
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            mod = __import__('shipa.commands.cmd_{}'.format(name), None, None, ["cli"])
        except ImportError as e:
            return
        return mod.cli


@click.command(cls=ShipaCLI, context_settings=CONTEXT_SETTINGS)
@click.option('-s', '--server', help='shipa server, for example http://shipa.org:8080', required=True)
@click.option('-e', '--email', help='user email')
@click.option('-p', '--password', help='user password')
@click.option('-t', '--token', help='token')
@click.option('--ca-cert', help='shipa ca certificate file')
@click.option('--ca', help='shipa ca certificate')
@click.option('--insecure', is_flag=True, help="enables ssl insecure mode")
@click.option('-v', '--verbose', is_flag=True, help="enables verbose mode")
@click.pass_context
def cli(ctx, server, email, password, token, ca_cert, ca, insecure, verbose):
    """A shipa command line interface."""

    with HttpClient(ca_cert=ca_cert, ca=ca, insecure=insecure) as http_client:
        client = ShipaClient(server=server, client=http_client, email=email, password=password,
                             verbose=verbose, token=token)
        ctx.obj = Environment(verbose=verbose, client=client)



CMD_APP_CREATE = 'create a new app'
OPT_APP_CREATE_DESCRIPTION = 'sets a description for your app'
OPT_APP_CREATE_POOL = 'defines which pool your app will be deployed'
OPT_APP_CREATE_TEAM = 'describes which team is responsible for the created app'
OPT_APP_CREATE_ROUTER = 'defines the router to be used'
OPT_APP_CREATE_PLAN = ' parameter defines the plan to be used'
OPT_APP_CREATE_TAG = 'sets a tag to your app. available multiple times'
OPT_APP_CREATE_DEPENDENCY = 'sets inspect file to create app dependencies. available multiple times'
OPT_APP_CREATE_ROUTER_opt = 'KEY=VALUE \n\n' \
                            'Allow passing custom parameters to the router used by the application\'s plan. ' \
                            'available multiple times'
CMD_APP_DEPLOY = 'deploy the source to shipa server'
OPT_APP_DEPLOY_APP = 'specify which app you want to deploy'
OPT_APP_DEPLOY_STEP_INTERVAL = 'single step duration. supported min: m, hour:h, second:s. ex. 1m, 60s, 1h'
OPT_APP_DEPLOY_STEPS = 'steps for canary deployment'
OPT_APP_DEPLOY_DIRECTORY = 'directory to deploy (default .)'
OPT_APP_DEPLOY_STEP_WEIGHT = 'deploy step weight'
CMD_APP_REMOVE = 'remove a shipa app'
OPT_APP_REMOVE_APP = 'specify which app you want to remove'
CMD_APP_MOVE = 'move a shipa app to another pool'
OPT_APP_MOVE_APP = 'specify which app you want to move'
OPT_APP_MOVE_POOL = 'specify which pool you want to move'
CMD_POOL_ADD = 'adds a new pool'
OPT_POOL_PUBLIC = 'make pool public (all teams can use it)'
OPT_POOL_DEFAULT = 'make pool default (when none is specified during app-create this pool will be used)'
OPT_POOL_PROVISIONER = 'provisioner associated to the pool (empty for default docker provisioner)'
OPT_POOL_PLAN = 'the plan when adding app to this pool'
OPT_POOL_QUOTA_LIMIT = 'quota limit for app when adding it to this pool'
OPT_POOL_NAMESPACE = 'kubernetes namespace for this pool'
OPT_POOL_DRIVER = 'node drivers accepted in pool. available multiple times'
CMD_POOL_REMOVE = 'remove an existing pool.'
CMD_POOL_UPDATE = 'updates attributes for a pool.'
CMD_NODE_ADD = 'creates or registers a new node in the cluster'
OPT_NODE_POOL = 'pool which will use this node. available multiple times'
OPT_NODE_REGISTER = 'registers an existing docker endpoint, the IaaS won\'t be called.'
OPT_NODE_CA_CERT = 'path to CA file shipa should trust.'
OPT_NODE_CLIENT_CERT = 'path to client TLS certificate file.'
OPT_NODE_CLIENT_KEY = 'path to client TLS key file.'
OPT_NODE_IAAS = 'which iaas provider should be used, if not set shipa will use the default ' \
                'iaas specified in shipa.conf file.'
OPT_NODE_IAASID = 'iaas\'s id.'
OPT_NODE_TEMPLATE = 'A machine template with predefined parameters, additional parameters will override ' \
                    'template ones.'
OPT_NODE_ADDRESS = 'only used if --register option is used. Should point to the endpoint of a working server.'
OPT_NODE_DRIVER = 'which node driver should be used. amazonec2, google, azure, vmwarevsphere, generic, ' \
                  'hyperv, openstack, virtualbox, vmwarefusion'
CMD_NODE_REMOVE = 'removes a node from the cluster'
CMD_NODE_DESTROY = 'destroy node from Iaas'
CMD_NODE_NO_REBALANCE = 'don\'t rebalance containers from removed node'
CMD_CLUSTER_ADD = 'creates a kubernetes cluster definition'
CMD_CLUSTER_ADDRESS = 'address to be used in cluster'
CMD_CLUSTER_TOKEN = 'token used for connection to the cluster'
OPT_CLUSTER_POOL = 'pool which will use this cluster'
OPT_CLUSTER_TEAM = 'cluster teams'
OPT_CLUSTER_DEFAULT = 'whether this is the default cluster.'
OPT_CLUSTER_INGRESS_SERVICE = 'specify ingress service type (LoadBalancer/ClusterIP/NodePort)'
OPT_CLUSTER_INGRESS_IP = 'ingress ip address'
OPT_CLUSTER_INGRESS_PORT = 'ingress port'
OPT_CLUSTER_CUSTOM = 'KEY=VALUE \n\n' \
                     'custom cluster specific data'
OPT_CLUSTER_PROVISIONER = 'represents a kubernetes cluster'
KUBERNETES_PROVISIONER = 'kubernetes'
OPT_CLUSTER_REMOVE = 'Removes a kubernetes cluster definition'



import click

from shipa.commands import constant


@click.group()
def cli_add():
    pass


@cli_add.command("add", short_help=constant.CMD_POOL_ADD)
@click.argument("pool", required=True)
@click.option('-d', '--default', is_flag=True, help=constant.OPT_POOL_DEFAULT)
@click.option('-p', '--public', is_flag=True, help=constant.OPT_POOL_PUBLIC)
@click.option('--accept-drivers', help=constant.OPT_POOL_DRIVER, multiple=True)
@click.option('--app-quota-limit', help=constant.OPT_POOL_QUOTA_LIMIT, type=int)
@click.option('--provisioner', help=constant.OPT_POOL_PROVISIONER)
@click.option('--plan', help=constant.OPT_POOL_PLAN)
@click.option('--kubernetes-namespace', help=constant.OPT_POOL_NAMESPACE)
@click.pass_obj
def create(env, pool, default, public, accept_drivers, app_quota_limit, provisioner, plan, kubernetes_namespace):
    """Each docker node added by shipa engine belongs to one pool."""

    env.client.auth()
    env.client.pool_add(pool=pool, default=default, public=public, accept_drivers=accept_drivers,
                        app_quota_limit=app_quota_limit, provisioner=provisioner, plan=plan,
                        kubernetes_namespace=kubernetes_namespace)
    print('Pool successfully registered.')


@click.group()
def cli_remove():
    pass


@cli_remove.command("remove", short_help=constant.CMD_POOL_REMOVE)
@click.argument("pool", required=True)
@click.pass_obj
def remove(env, pool):
    """Remove an existing pool."""

    env.client.auth()
    env.client.pool_remove(pool=pool)
    print('Pool successfully removed.')


@click.group()
def cli_update():
    pass


@cli_update.command("update", short_help=constant.CMD_POOL_UPDATE)
@click.argument("pool", required=True)
@click.option('-d', '--default', is_flag=True, help=constant.OPT_POOL_DEFAULT)
@click.option('-p', '--public', is_flag=True, help=constant.OPT_POOL_PUBLIC)
@click.option('--accept-drivers', help=constant.OPT_POOL_DRIVER, multiple=True)
@click.option('--app-quota-limit', help=constant.OPT_POOL_QUOTA_LIMIT, type=int)
@click.option('--plan', help=constant.OPT_POOL_PLAN)
@click.pass_obj
def update(env, pool, default, public, accept_drivers, app_quota_limit, plan):
    """Updates attributes for a pool."""

    env.client.auth()
    env.client.pool_update(pool=pool, default=default, public=public, accept_drivers=accept_drivers,
                           app_quota_limit=app_quota_limit, plan=plan)
    print('Pool successfully updated.')


cli = click.CommandCollection(sources=[cli_add, cli_remove, cli_update])



import click

from shipa.commands import constant
from shipa.commands.constant import KUBERNETES_PROVISIONER
from shipa.utils import validate_map


@click.group()
def cli_add():
    pass


@cli_add.command("add", short_help=constant.CMD_CLUSTER_ADD)
@click.argument("name", required=True)
@click.option('-a', '--address', help=constant.CMD_CLUSTER_ADDRESS, multiple=True)
@click.option('-d', '--default', is_flag=True, help=constant.OPT_CLUSTER_DEFAULT)
@click.option('-p', '--pools', help=constant.OPT_CLUSTER_POOL, multiple=True)
@click.option('-t', '--teams', help=constant.OPT_CLUSTER_TEAM, multiple=True)
@click.option('--token', help=constant.CMD_CLUSTER_TOKEN)
@click.option('--ingress-service', help=constant.OPT_CLUSTER_INGRESS_SERVICE)
@click.option('--ingress-ip', help=constant.OPT_CLUSTER_INGRESS_IP)
@click.option('--ingress-port', help=constant.OPT_CLUSTER_INGRESS_PORT, type=int, default=0)
@click.option('--cacert', help=constant.OPT_NODE_CA_CERT)
@click.option('--clientcert', help=constant.OPT_NODE_CLIENT_CERT)
@click.option('--clientkey', help=constant.OPT_NODE_CLIENT_KEY)
@click.option('--provisioner', help=constant.OPT_CLUSTER_PROVISIONER, default=KUBERNETES_PROVISIONER, show_default=True)
@click.option('--custom-data', help=constant.OPT_CLUSTER_CUSTOM, callback=validate_map, multiple=True)
@click.pass_obj
def add(env, name, address, default, pools, teams, token, ingress_service, ingress_ip, ingress_port, cacert,
        clientcert, clientkey, provisioner, custom_data):
    """Creates a kubernetes cluster definition."""

    env.client.auth()
    env.client.cluster_add(name=name, address=address, default=default, pools=pools, teams=teams, token=token,
                           ingress_service=ingress_service, ingress_ip=ingress_ip, ingress_port=ingress_port,
                           ca_cert=cacert, client_cert=clientcert, client_key=clientkey, provisioner=provisioner,
                           custom_data=custom_data)
    print('Cluster successfully added.')


@click.group()
def cli_remove():
    pass


@cli_remove.command("remove", short_help=constant.OPT_CLUSTER_REMOVE)
@click.argument("name", required=True)
@click.pass_obj
def remove(env, name):
    """Removes a kubernetes cluster definition."""

    env.client.auth()
    env.client.cluster_remove(name=name)
    print('Cluster {0} has been removed!'.format(name))


cli = click.CommandCollection(sources=[cli_add, cli_remove])



import os
import tarfile
import tempfile
import click

from shipa.gitignore import GitIgnore


class RepositoryFolder(object):
    IGNORE_FILENAME = '.shipaignore'

    def __init__(self, directory, verbose=False):
        assert directory is not None
        assert verbose is not None

        self.directory = directory
        self.verbose = verbose

        ignore_path = os.path.join(directory, self.IGNORE_FILENAME)
        lines = None
        if os.path.isfile(ignore_path) is True:
            with open(ignore_path, 'r') as f:
                lines = f.readlines()
        self.shipa_ignore = GitIgnore(lines or [])

    def create_tarfile(self):

        os.chdir(self.directory)
        if self.verbose:
            print('Create tar archive:')

        def filter(info):
            if info.name.startswith('./.git'):
                return

            filename = info.name[2:]

            if self.shipa_ignore.match(filename):
                if self.verbose:
                    print('IGNORE: ', filename)
                return

            if self.verbose:
                print('OK', filename)
            return info

        f = tempfile.TemporaryFile(suffix='.tar.gz')
        tar = tarfile.open(fileobj=f, mode="w:gz")
        tar.add(name='.',
                recursive=True,
                filter=filter)
        tar.close()
        f.seek(0)
        return f


def parse_step_interval(step_interval):
    if step_interval.endswith('s'):
        return int(step_interval[:len(step_interval) - 1])
    elif step_interval.endswith('m'):
        return int(step_interval[:len(step_interval) - 1]) * 60
    elif step_interval.endswith('h'):
        return int(step_interval[:len(step_interval) - 1]) * 60 * 60
    elif step_interval == '':
        return 1
    else:
        return step_interval


def validate_map(ctx, param, values):
    try:
        map_data = dict()
        for value in values:
            k, v = value.split('=')
            map_data[k] = v
        return map_data
    except ValueError:
        raise click.BadParameter('need to be in format KEY=VALUE')



import unittest

from shipa.client.client import ShipaClient, CONST_TEST_TOKEN, CONST_TEST_SERVER
from shipa.client.http import MockResponse, MockClient


class ShipaAppTestCase(unittest.TestCase):

    def test_app_create(self):
        try:
            response = MockResponse(code=201, text='{"status": "success", "repository_url": "repository_url"}')
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.app_create(appname='test-app', pool="test", team="admin", platform="python")

        except Exception as e:
            assert e is None, str(e)

    def test_app_create_failed_with_error_code(self):
        response = MockResponse(code=400, text='{"status": "success", "repository_url": "repository_url"}')
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_create(appname='test-app', pool="test", team="admin", platform="python")

    def test_app_create_failed_with_error_status(self):
        response = MockResponse(code=201, text='{"status": "failed", "repository_url": "repository_url"}')
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_create(appname='test-app', pool="test", team="admin", platform="python")

    def test_app_remove(self):
        try:
            response = MockResponse(code=200, text='{"Message": "removed a shipa app"}')
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.app_remove(appname='test-app')

        except Exception as e:
            assert e is None, str(e)

    def test_app_remove_failed(self):
        response = MockResponse(code=400, text='{"Message": "failed to remove a shipa app"}')
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_remove(appname='test-app')

    def test_app_deploy(self):
        try:
            response = MockResponse(code=200, text="deployed a app\n OK\n")
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.app_deploy(appname='test-app')

        except Exception as e:
            assert e is None, str(e)

    def test_app_deploy_failed(self):
        response = MockResponse(code=200, text="deployed a app\n FAIL\n")
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_deploy(appname='test-app')

    def test_app_deploy_failed_without_response_text(self):
        response = MockResponse(code=200)
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_deploy(appname='test-app')

    def test_app_move(self):
        try:
            response = MockResponse(code=200, text="moved a app\n OK\n")
            http = MockClient(response=response)
            client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)
            client.app_move(appname='test-app', pool='test-pool')

        except Exception as e:
            assert e is None, str(e)

    def test_app_move_failed(self):
        response = MockResponse(code=200, text="moved a app\n FAIL\n")
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_move(appname='test-app', pool='test-pool')

    def test_app_move_failed_without_response_text(self):
        response = MockResponse(code=200)
        http = MockClient(response=response)
        client = ShipaClient(server=CONST_TEST_SERVER, client=http, token=CONST_TEST_TOKEN)

        with self.assertRaises(Exception):
            client.app_move(appname='test-app', pool='test-pool')




"""A CI tool to deploy applications to Shipa instances
See:
https://github.com/shipa-corp/ci-integration
"""

from setuptools import setup

description = 'A CI tool to deploy applications to Shipa instances'
url = 'https://github.com/shipa-corp/ci-integration'

setup(
    name='shipa-ci',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    description=description,
    long_description=
    description + '\n\n' + url,
    url=url,
    packages=["shipa", "shipa.commands"],
    include_package_data=True,
    scripts=['shipa-ci'],
    install_requires=['requests>=2.22.0', 'click>=7.1.2', 'schematics>=2.1.0'],
    classifiers=[
        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',

        'Operating System :: OS Independent',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points="""
        [console_scripts]
        shipa=shipa.cli:cli
    """,
    license='MIT',
    keywords='shipa-ci',
    platforms=['MacOS', 'Debian', 'Fedora', 'CentOS']
)



import unittest

from shipa.gitignore import get_pattern_from_line, GitIgnore


class GetPatternFromLineTestCase(unittest.TestCase):

    def test_negative(self):
        _, negative = get_pattern_from_line(line='!env3')
        self.assertTrue(negative)
        _, negative = get_pattern_from_line(line='    !env3')
        self.assertTrue(negative)
        _, negative = get_pattern_from_line(line='    !  env3')
        self.assertTrue(negative)

    def test_comments(self):
        pattern, negative = get_pattern_from_line(line='#env3')
        self.assertIsNone(pattern)
        pattern, negative = get_pattern_from_line(line='  #env3')
        self.assertIsNone(pattern)
        pattern, negative = get_pattern_from_line(line='\#env3')
        self.assertIsNotNone(pattern)


class GitIgnoreTestCase(unittest.TestCase):
    def test_ignore_lines(self):
        lines = ['abc/def', 'a/b/c', 'b']
        ignore = GitIgnore(lines)

        #  Paths which are targeted by the above 'lines'
        assert ignore.match('abc/def/child') is True, 'abc/def/child should match'
        assert ignore.match('a/b/c/d') is True, 'a/b/c/d should match'

        #  Paths which are not targeted by the above 'lines'
        assert ignore.match('abc') is False, 'abc should not match'
        assert ignore.match('def') is False, 'def should not match'
        assert ignore.match('bd') is False, 'bd should not match'

    def test_empty(self):
        ignore = GitIgnore([])

        assert ignore.match('a') is False, 'should not match any path'
        assert ignore.match('a/b') is False, 'should not match any path'
        assert ignore.match('.foobar') is False, 'should not match any path'

    def test_exclude_everything(self):
        ignore = GitIgnore([
            '# exclude everything except directory foo/bar',
            '/*',
            '!/foo',
            '/foo/*',
            '!/foo/bar'
        ])
        assert ignore.match('a') is True, 'a should match'
        assert ignore.match('foo/baz') is True, 'foo/baz should match'
        assert ignore.match('foo') is False, 'foo should not match'
        assert ignore.match('/foo/bar') is False, '/foo/bar should not match'

    def test_handle_spaces(self):
        ignore = GitIgnore([
            '#',
            '# A comment',
            '           ',
            '    # Another comment',
            '             ',
            '# Invalid Comment',
            'abc/def',
        ])

        self.assertEqual(1, len(ignore.items))
        assert ignore.match('abc/abc') is False, '/abc/abc should not match'
        assert ignore.match('abc/def') is True, '/abc/def should match'

    def test_handle_leading_slashes(self):
        ignore = GitIgnore([
            '/a/b/c',
            'd/e/f',
            '/g',
        ])

        self.assertEqual(3, len(ignore.items))
        assert ignore.match('a/b/c'), 'a/b/c should match'
        assert ignore.match('a/b/c/d'), 'a/b/c/d should match'
        assert ignore.match('d/e/f'), 'd/e/f should match'
        assert ignore.match('g'), 'g should match'

    def test_handle_leading_special_chars(self):
        ignore = GitIgnore([
            '# Comment',
            '\#file.txt',
            '\!file.txt',
            'file.txt',
        ])

        assert ignore.match('#file.txt'), '#file.txt should match'
        assert ignore.match('!file.txt'), '!file.txt should match'
        assert ignore.match('a/!file.txt'), 'a/!file.txt should match'
        assert ignore.match('file.txt'), 'file.txt should match'
        assert ignore.match('a/file.txt'), 'a/file.txt should match'
        assert not ignore.match('file2.txt'), 'file2.txt should not match'

    def test_handle_all_files_in_dir(self):
        ignore = GitIgnore(['Documentation/*.html'])

        assert ignore.match('Documentation/git.html'), 'Documentation/git.html should match'
        assert not ignore.match('Documentation/ppc/ppc.html'), 'Documentation/ppc/ppc.html should not match'
        assert ignore.match('tools/perf/Documentation/perf.html'), 'tools/perf/Documentation/perf.html should not match'

    def test_handle_double_star(self):
        ignore = GitIgnore(['**/foo', 'bar'])

        assert ignore.match('foo'), 'foo should match'
        assert ignore.match('baz/foo'), 'baz/foo should match'
        assert ignore.match('bar'), 'bar should match'
        assert ignore.match('baz/bar'), 'baz/bar should match'

    def test_handle_leading_slash_path(self):
        ignore = GitIgnore(['/*.c'])

        assert ignore.match('hello.c'), 'hello.c should match'
        assert not ignore.match('foo/hello.c'), 'foo/hello.c should not match'

    def test_nested_dot_files(self):
        ignore = GitIgnore([
            '**/external/**/*.md',
            '**/external/**/*.json',
            '**/external/**/*.gzip',
            '**/external/**/.*ignore',

            '**/external/foobar/*.css',
            '**/external/barfoo/less',
            '**/external/barfoo/scss',
        ])
        assert ignore.match('external/foobar/angular.foo.css'), 'external/foobar/angular.foo.css'
        assert ignore.match('external/barfoo/.gitignore'), 'external/barfoo/.gitignore'
        assert ignore.match('external/barfoo/.bower.json'), 'external/barfoo/.bower.json'

    def test_wild_card_files(self):
        ignore = GitIgnore(['*.swp', '/foo/*.wat', 'bar/*.txt'])

        assert ignore.match('yo.swp'), 'should ignore all swp files'
        assert ignore.match('something/else/but/it/hasyo.swp'), 'should ignore all swp files in other directories'

        assert ignore.match('foo/bar.wat'), 'should ignore all wat files in foo - nonpreceding /'
        assert ignore.match('/foo/something.wat'), 'should ignore all wat files in foo - preceding /'

        assert ignore.match('bar/something.txt'), 'should ignore all txt files in bar - nonpreceding /'
        assert ignore.match('/bar/somethingelse.txt'), 'should ignore all txt files in bar - preceding /'

        assert not ignore.match('something/not/infoo/wat.wat'), 'wat files should only be ignored in foo'
        assert not ignore.match('something/not/infoo/wat.txt'), 'txt files should only be ignored in bar'

    def test_preceding_slash(self):
        ignore = GitIgnore(['/foo', 'bar/'])

        assert ignore.match('foo/bar.wat'), 'should ignore all files in foo - nonpreceding /'
        assert ignore.match('/foo/something.txt'), 'should ignore all files in foo - preceding /'

        assert ignore.match('bar/something.txt'), 'should ignore all files in bar - nonpreceding /'
        assert ignore.match('/bar/somethingelse.go'), 'should ignore all files in bar - preceding /'
        assert ignore.match('/boo/something/bar/boo.txt'), 'should block all files if bar is a sub directory'

        assert not ignore.match(
            'something/foo/something.txt'), 'should only ignore top level foo directories- not nested'



import json
import click
import requests

from schematics.exceptions import ValidationError, DataError
from shipa.client.types import AddNodeRequest, UpdatePoolRequest, AddPoolRequest, AddAppRequest, DeployAppRequest, \
    MoveAppRequest, RemovePoolRequest, AddClusterRequest
from shipa.commands.constant import KUBERNETES_PROVISIONER
from shipa.utils import RepositoryFolder, parse_step_interval

CONST_TEST_TOKEN = "test-token"
CONST_TEST_SERVER = "test-server"


class ShipaClient(object):

    def __init__(self, server, client, email=None, password=None, token=None, verbose=False):
        self.server = server
        if not server.startswith('http'):
            self.urlbase = 'http://{0}'.format(server)
        else:
            self.urlbase = server

        self.email = email
        self.password = password
        self.token = token
        self.verbose = verbose
        self.http = client

        if token is not None:
            self.headers = {"Authorization": "bearer " + self.token}
            self.json_headers = {"Authorization": "bearer " + self.token, 'Content-Type': 'application/json',
                                 'Accept': 'application/json'}

    def print_response(self, response):
        if self.verbose:
            print(response.text)
            print(response.status_code)

    def auth(self):
        if self.token is not None:
            return

        if self.email is None or self.password is None:
            raise click.ClickException('Please, provide email and password')

        url = '{0}/users/{1}/tokens'.format(self.urlbase, self.email)
        params = {'password': self.password}
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}

        try:
            r = self.http.post(url, params=params, headers=headers)
            self.print_response(r)

            if r.status_code != 200:
                raise click.ClickException('Invalid token or user/password ({0})'.format(r.text.strip("\n")))

            self.token = r.json()['token']
            self.headers = {"Authorization": "bearer " + self.token}
            self.json_headers = {"Authorization": "bearer " + self.token, 'Content-Type': 'application/json',
                                 'Accept': 'application/json'}

        except (requests.ConnectionError, IOError) as e:
            raise click.ClickException(str(e))

    def app_deploy(self, appname, directory='.', steps=1, step_interval='1s', step_weight=300):
        files = None
        try:
            url = '{0}/apps/{1}/deploy'.format(self.urlbase, appname)

            if self.server is not CONST_TEST_SERVER:
                folder = RepositoryFolder(directory, verbose=self.verbose)
                file = folder.create_tarfile()
                files = {'file': file}

            request = DeployAppRequest({'steps': steps, 'step_interval': parse_step_interval(step_interval),
                                        'step_weight': step_weight})
            request.validate()

            r = self.http.post(url, files=files, headers=self.headers, data=request.to_primitive())
            self.print_response(r)
            if r.text is None:
                raise click.ClickException(r.text)

            ok = any(line.strip() == "OK" for line in r.text.split('\n'))

            if ok is False:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def app_create(self, appname, team, pool, platform=None, description=None, dependency_files=tuple(), tags=tuple(),
                   plan=None, router=None, router_opts=dict()):
        try:
            url = '{0}/apps'.format(self.urlbase)

            request = AddAppRequest({'name': appname, 'platform': platform, 'plan': plan, 'teamOwner': team,
                                     'description': description, 'pool': pool, 'router': router,
                                     'dependency_files': dependency_files, 'tags': tags,
                                     'router_opts': router_opts})
            request.validate()

            r = self.http.post(url, headers=self.json_headers, data=json.dumps(request.to_primitive()))
            self.print_response(r)
            if r.status_code != 201:
                raise click.ClickException(r.text)

            out = json.loads(r.text)
            if out['status'] != 'success':
                raise click.ClickException(out['status'])

            print('App {0} has been created!'.format(appname))
            print('Use app-info to check the status of the app and its units')
            if out['repository_url'] is not None:
                print('Your repository for {0} project is {1}'.format(appname, out['repository_url']))

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def app_remove(self, appname):
        try:
            url = '{0}/apps/{1}'.format(self.urlbase, appname)

            r = self.http.delete(url, headers=self.headers)
            self.print_response(r)
            if r.status_code != 200:
                raise click.ClickException(r.text)

            responses = r.text.split('\n')
            for response in responses:
                if response.find("Message") < 0:
                    continue
                out = json.loads(response)
                print(out['Message'].replace('\n', ''))

        except requests.ConnectionError as e:
            raise click.ClickException(str(e))
        except AttributeError:
            print('Done removing application.')

    def autoscale_check(self):
        try:
            print('running autoscale checks')
            url = '{0}/node/autoscale/run'.format(self.urlbase)

            r = self.http.post(url, headers=self.headers)
            self.print_response(r)
            if r.text is None:
                raise click.ClickException(r.text)

            if "Node Autoscaler available only" in r.text:
                raise click.ClickException(r.text)

            responses = r.text.split('\n')
            for response in responses:
                if response.find("Message") < 0:
                    continue
                out = json.loads(response)
                print(out['Message'].replace('\n', ''))

        except requests.ConnectionError as e:
            raise click.ClickException(str(e))

    def app_move(self, appname, pool):
        url = '{0}/apps/{1}/move'.format(self.urlbase, appname)

        try:
            request = MoveAppRequest({'pool': pool})
            request.validate()

            r = self.http.post(url, headers=self.headers, data=request.to_primitive())
            self.print_response(r)
            if r.text is None:
                raise click.ClickException(r.text)

            ok = any(line.strip() == "OK" for line in r.text.split('\n'))
            if ok is False:
                raise click.ClickException(r.text.replace('\n\n', ''))

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def pool_add(self, pool, default=False, public=False, accept_drivers=tuple(), app_quota_limit=None,
                 provisioner=None, plan=None, kubernetes_namespace=None):

        try:
            url = '{0}/pools'.format(self.urlbase)

            request = AddPoolRequest({'name': pool, 'public': public, 'default': default,
                                      'provisioner': provisioner, 'plan': plan, 'accept_drivers': accept_drivers,
                                      'kubernetes_namespace': kubernetes_namespace, 'app_quota_limit': app_quota_limit})
            request.validate()

            r = self.http.post(url, headers=self.headers, data=request.to_primitive())
            self.print_response(r)
            if r.status_code != 201:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def pool_remove(self, pool):

        try:
            url = '{0}/pools/{1}'.format(self.urlbase, pool)

            r = self.http.delete(url, headers=self.headers)
            self.print_response(r)
            if r.status_code != 200:
                raise click.ClickException(r.text)

        except requests.ConnectionError as e:
            raise click.ClickException(str(e))

    def pool_update(self, pool, default=False, public=False, accept_drivers=tuple(), app_quota_limit=None, plan=None):
        try:
            url = '{0}/pools/{1}'.format(self.urlbase, pool)
            request = UpdatePoolRequest({'public': public, 'default': default, 'plan': plan,
                                         'app_quota_limit': app_quota_limit, 'accept_drivers': accept_drivers})
            request.validate()

            r = self.http.put(url, headers=self.headers, data=request.to_primitive())
            self.print_response(r)
            if r.status_code != 200:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def file_read(self, path=None):
        try:
            if path is not None:
                with open(path, "r") as f:
                    return f.read()
            return None
        except IOError as e:
            raise click.ClickException(e)

    def node_add(self, pools=tuple(), iaas=None, iaasid=None, address=None, template=None, driver=None,
                 cacert=None, clientcert=None, clientkey=None, register=False):

        try:
            url = '{0}/node'.format(self.urlbase)

            request = AddNodeRequest({'register': register, 'pool': pools[0], 'alternative_pools': pools[1:],
                                      'client_key': self.file_read(clientkey),
                                      'client_cert': self.file_read(clientcert), 'ca_cert': self.file_read(cacert),
                                      'meta_data': {'driver': driver, 'address': address, 'iaas': iaas,
                                                    'iaas_id': iaasid, 'template': template}})
            request.validate()

            r = self.http.post(url, headers=self.json_headers, data=json.dumps(request.to_primitive()))
            self.print_response(r)
            if r.status_code != 201:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def node_remove(self, address, no_rebalance=False, destroy=False):
        try:
            url = '{0}/node'.format(self.urlbase, address)

            request = RemovePoolRequest({'address': address, 'destroy': destroy, 'no_rebalance': no_rebalance})
            request.validate()

            r = self.http.delete(url, headers=self.headers, params=request.to_primitive())
            self.print_response(r)
            if r.status_code != 200:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def cluster_add(self, name, address=tuple(), default=None, pools=tuple(), teams=tuple(), token=None,
                    ingress_service=None, ingress_ip=None, ingress_port=0, ca_cert=None,
                    client_cert=None, client_key=None, provisioner=KUBERNETES_PROVISIONER,
                    custom_data=dict()):

        try:
            url = '{0}/provisioner/clusters'.format(self.urlbase)

            request = AddClusterRequest({'name': name, 'address': address, 'default': default, 'teams': teams,
                                         'token': token, 'ingress_service': ingress_service, 'ingress_ip': ingress_ip,
                                         'ingress_port': ingress_port, 'ca_cert': ca_cert, 'pools': pools,
                                         'client_cert': client_cert, 'client_key': client_key,
                                         'provisioner': provisioner, 'custom_data': custom_data})
            request.validate()

            r = self.http.post(url, headers=self.json_headers, data=json.dumps(request.to_primitive()))
            self.print_response(r)
            if r.status_code != 201:
                raise click.ClickException(r.text)

        except (requests.ConnectionError, ValidationError, DataError) as e:
            raise click.ClickException(str(e))

    def cluster_remove(self, name):
        try:
            url = '{0}/provisioner/clusters/{1}'.format(self.urlbase, name)

            r = self.http.delete(url, headers=self.headers)
            self.print_response(r)
            if r.status_code != 200:
                raise click.ClickException(r.text)

        except requests.ConnectionError as e:
            raise click.ClickException(str(e))



import click

from shipa.commands import constant


@click.group()
def cli_add():
    pass


@cli_add.command("add", short_help=constant.CMD_NODE_ADD)
@click.option('-p', '--pool', help=constant.OPT_NODE_POOL, multiple=True, required=True)
@click.option('--iaas', help=constant.OPT_NODE_IAAS, default="dockermachine", show_default=True)
@click.option('--iaas-id', help=constant.OPT_NODE_IAASID)
@click.option('--address', help=constant.OPT_NODE_ADDRESS, required=True)
@click.option('--template', help=constant.OPT_NODE_TEMPLATE)
@click.option('--driver', help=constant.OPT_NODE_DRIVER, default="generic", show_default=True)
@click.option('--cacert', help=constant.OPT_NODE_CA_CERT)
@click.option('--clientcert', help=constant.OPT_NODE_CLIENT_CERT)
@click.option('--clientkey', help=constant.OPT_NODE_CLIENT_KEY)
@click.option('--register', is_flag=True, help=constant.OPT_NODE_REGISTER)
@click.pass_obj
def add(env, pool, iaas, iaas_id, address, template, driver, cacert, clientcert, clientkey, register):
    """Creates or registers a new node in the cluster"""

    env.client.auth()
    env.client.node_add(pools=pool, iaas=iaas, iaasid=iaas_id, address=address, template=template, driver=driver,
                        cacert=cacert, clientcert=clientcert, clientkey=clientkey, register=register)
    print('Node successfully added.')


@click.group()
def cli_remove():
    pass


@cli_add.command("remove", short_help=constant.CMD_NODE_REMOVE)
@click.argument("address", required=True)
@click.option('--no-rebalance', is_flag=True, help=constant.CMD_NODE_NO_REBALANCE)
@click.option('--destroy', is_flag=True, help=constant.CMD_NODE_DESTROY)
@click.pass_obj
def remove(env, address, no_rebalance, destroy):
    """Removes a node from the cluster"""

    env.client.auth()
    env.client.node_remove(address=address, no_rebalance=no_rebalance, destroy=destroy)
    print('Node successfully removed.')


cli = click.CommandCollection(sources=[cli_add, cli_remove])
